/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.ts.service;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.http.HttpService;
import ru.ulstu.extractor.http.JsonTimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesType;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.ts.repository.TimeSeriesRepository;
import ru.ulstu.extractor.ts.repository.TimeSeriesValueRepository;
import ru.ulstu.extractor.ts.util.TimeSeriesDateMapper;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

@Service
public class TimeSeriesService {
    private final static Logger LOG = LoggerFactory.getLogger(TimeSeriesService.class);
    private final TimeSeriesRepository timeSeriesRepository;
    private final TimeSeriesValueRepository timeSeriesValueRepository;
    private final TimeSeriesDateMapper.TimeSeriesInterval timeSeriesInterval = TimeSeriesDateMapper.TimeSeriesInterval.HOUR;
    private final HttpService httpService;
    private final static String TIME_SERIES_SERVICE_URL = "http://time-series.athene.tech/api/1.0/add-time-series?setKey=git-extractor";

    public TimeSeriesService(TimeSeriesRepository timeSeriesRepository,
                             TimeSeriesValueRepository timeSeriesValueRepository,
                             HttpService httpService) {
        this.timeSeriesRepository = timeSeriesRepository;
        this.timeSeriesValueRepository = timeSeriesValueRepository;
        this.httpService = httpService;
    }

    @Transactional
    public TimeSeries save(String timeSeriesName, TimeSeriesType timeSeriesType, List<TimeSeriesValue> timeSeriesValues) {
        LOG.debug("Start save {} time series with {} time series values ", timeSeriesName, timeSeriesValues.size());
        final TimeSeries timeSeries = findOrCreate(timeSeriesName);
        List<TimeSeriesValue> timeSeriesValuesToRemove = timeSeries.getValues();
        timeSeries.setValues(timeSeriesValues);
        timeSeries.setTimeSeriesType(timeSeriesType);
        LOG.debug("Save time series {} ", timeSeries.getName());
        TimeSeries savedTimeSeries = timeSeriesRepository.save(timeSeries);
        LOG.debug("Clear {} time series values ", timeSeriesValuesToRemove.size());
        timeSeriesValueRepository.deleteAll(timeSeriesValuesToRemove);
        sendToTimeSeriesService(savedTimeSeries);
        return savedTimeSeries;
    }

    public TimeSeries findOrCreate(String timeSeriesName) {
        Optional<TimeSeries> maybeTimeSeries = timeSeriesRepository.findByName(timeSeriesName);
        if (maybeTimeSeries.isPresent()) {
            LOG.debug("TimeSeries {} exists.", maybeTimeSeries.get().getName());
            return maybeTimeSeries.get();
        }
        return timeSeriesRepository.save(new TimeSeries(timeSeriesName));
    }

    /**
     * Сохранить список временных рядов
     *
     * @param timeSeries список ВР
     * @return сохраненный список ВР
     */
    @Transactional
    public List<TimeSeries> save(List<TimeSeries> timeSeries) {
        List<TimeSeries> results = new ArrayList<>();
        for (TimeSeries ts : timeSeries) {
            results.add(save(ts.getName(), ts.getTimeSeriesType(), ts.getValues()));
        }
        return results;
    }

    public TimeSeriesDateMapper.TimeSeriesInterval getTimeSeriesInterval() {
        return timeSeriesInterval;
    }

    private void sendToTimeSeriesService(TimeSeries timeSeries) {
        new Thread(() -> {
            try {
                httpService.post(TIME_SERIES_SERVICE_URL, new JSONObject(new JsonTimeSeries(timeSeries)));
                LOG.debug("Успешно отправлен на сервис");
            } catch (Exception ex) {
                LOG.debug(ex.getMessage());
            }
        }).start();
    }

    public List<TimeSeries> getAllTimeSeries() {
        return timeSeriesRepository.findAll();
    }

    public TimeSeries getById(Integer tsId) {
        return timeSeriesRepository.findById(tsId)
                .orElseThrow(() -> new RuntimeException("Time series not found by id " + tsId));
    }

    public List<TimeSeriesType> getAllTimeSeriesTypes() {
        return Arrays.asList(TimeSeriesType.values());
    }
}
